/*
	BELLFORD.C										mayo / 2001
	Programa que hace las tablas de enrutamiento con el algoritmo de Bellman-Ford con auxilio de un
	programa servidor
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "winsock.h"

#define WINSOCK_VER 0x0101
#define DEF_PROTO 0
#define RCV_FLG 0
#define SND_FLG 0
#define PROG_NAME "Bellman Ford - Cliente"

const int INF=127;
const int MAXMAQS=8;
const int MAQNUM=1;
const struct in_addr IPADD={(unsigned char)148,			//mi direccion IP
														(unsigned char)204,
														(unsigned char)183,
														(unsigned char)166};
struct Ruta
{
	int cost;
	char rout[MAXMAQS];
};
int tengotab[MAXMAQS]={0};						//banderas de si tengo o no tablas
int dij[MAXMAQS][MAXMAQS];						//tablas
int numsaltos=0;

void terminate(int);
int bf(struct Ruta *);
int writetofile(struct Ruta *);

void main(void)
{
	WSADATA wsadata;
	struct Ruta ruta[MAXMAQS];
	int i, j;

	if(WSAStartup(WINSOCK_VER,&wsadata))
	{
		MessageBox(NULL,"Error en la version de Winsock!",PROG_NAME,MB_OK|MB_ICONEXCLAMATION);
		terminate(1);
	}
	/*Bellman-Ford*/
	if(bf(ruta))
		terminate(1);
	/*Resultado*/
	if(writetofile(ruta))
		terminate(1);
	MessageBox(NULL,"Listo!!",PROG_NAME,MB_OK|MB_ICONINFORMATION);
	WSACleanup();
}

/*Termina el programa*/
void terminate(int excd)
{
	WSACleanup();
	exit(excd);
}

/*Calcula la tabla de enrutamiento con el algoritmo de Bellman-Ford*/
int bf(struct Ruta *r)
{
	int gettabla(char *, int *);

	struct Ruta ra[MAXMAQS];
	char aux[2]={0,0}, dat;
	int n, j;

	//Inicializacin
	for(n=0; n<MAXMAQS; n++)
	{
		r[n].cost=ra[n].cost=INF;	//como si no hubiera ninguna conexion
		for(j=0;j<MAXMAQS;j++)
		{
			r[n].rout[j]=ra[n].rout[j]='\x0';	//no hay rutas aun
			dij[n][j]=INF;										//no hay tablas aun
		}
	}
	r[MAQNUM-1].cost=0;								//el costo a mi maquina es 0
	r[MAQNUM-1].rout[0]=MAQNUM;				//la ruta a mi maquina es ir a mi mismo

	while(memcmp(r, ra, sizeof(struct Ruta)*MAXMAQS))	//mientras ruta!=ranterior
	{
		numsaltos++;
		memcpy(ra,r,sizeof(struct Ruta)*MAXMAQS);				//backup de r en ranterior
		for(n=0; n<MAXMAQS; n++)		//llegar al nodo j pasando por el nodo n
		{
			if(r[n].cost==INF)
				continue;
			if(tengotab[n]==0)									//si no hay tabla del nodo n:
			{
				if(gettabla(r[n].rout,dij[n]))		//se pide la tabla
					return 1;
				tengotab[n]=1;										//y se actualiza la bandera
			}
			for(j=0; j<MAXMAQS; j++)	//para llegar a todos los nodos j
      {
				if(r[j].cost>ra[n].cost+dij[n][j])		//el costo que ya hay de cada
				{																//nodo es peor a ir por n y llegar a j
					dat=r[j].rout[strlen(r[j].rout)-1];	//ultimo elemento de la ruta

					r[j].cost=ra[n].cost+dij[n][j];			//actualizamos el costo
					strcpy(r[j].rout,r[n].rout);				//ruta es ahora ir al nodo n...
					if(dat!=j)													//y se agrega el nodo j si aun
					{																			//no estaba en la ruta
						aux[0]=j+1;
						strcat(r[j].rout,aux);
					}
				}
			}
		}
	}

	return 0;
}

/*Pide latabla del nodo n, pidiendosela a su servidor*/
int gettabla(char *tray, int *t)
{
	SOCKET sock;
	SOCKADDR_IN sockadd, to;
	char mess[1024], buff1[8], buff2[8];
	int tolen=sizeof(to), i, j, node;

	/*Inicializacion del socket*/
	sock=socket(PF_INET,SOCK_DGRAM,DEF_PROTO);	//creamos el socket
	if(sock==INVALID_SOCKET)
	{
		MessageBox(NULL,"Error al crear el socket!",PROG_NAME,MB_OK|MB_ICONSTOP);
		return 1;
	}
	sockadd.sin_family=AF_INET;					//definimos los parametros del socket
	sockadd.sin_port=0;
	sockadd.sin_addr.S_un.S_addr=0;
	if(bind(sock,(struct sockaddr far *)&sockadd,sizeof(SOCKADDR_IN)) == SOCKET_ERROR)
	{											//atamos el socket
		MessageBox(NULL,"Error al atar el socket!",PROG_NAME,MB_OK|MB_ICONSTOP);
		closesocket(sock);
		return 1;
	}
	//estructura del servidor en MI maquina
	to.sin_family=AF_INET;
	to.sin_port=htons(1000+MAQNUM);
	to.sin_addr=IPADD;

	/*Armado del mensaje a enviar*/
	//Formato del mensaje: N1-N2-...-T, donde: Nn=nodos posteriores al mio, T=pide la tabla
	mess[0]='\0';
	if(tray[0]=='\0' || tray[0]!=MAQNUM)	//el 1er elemento de la ruta es mi propia maquina
	{
		MessageBox(NULL,"Las rutas no comienzan con mi nodo!",PROG_NAME,
							MB_OK|MB_ICONSTOP);
		closesocket(sock);
		return 2;
	}
	for(i=1; tray[i]!='\0'; i++)
	{
		strcat(mess,itoa(tray[i],buff1,10));//metemos uno a uno la ruta para llegar a ese nodo
		strcat(mess,"-");	//separados por guiones
	}
	strcat(mess,"T");		//al final, la peticion de la tabla
	/*Mandamos el mensaje al servidor y esperamos la respuesta*/
	if(sendto(sock,(char far *)mess,sizeof(mess),SND_FLG,
						(struct sockaddr far *)&to,tolen) == SOCKET_ERROR)
	{
		MessageBox(NULL,"Error al enviar peticion al servidor!",PROG_NAME,MB_OK|MB_ICONSTOP);
		closesocket(sock);
		return 1;
	}
	if(recvfrom(sock,(char far *)mess,sizeof(mess),RCV_FLG,
							(struct sockaddr far *)&to,(int far *)&tolen) == SOCKET_ERROR)
	{
		MessageBox(NULL,"Error al recibir la respuesta del servidor!",PROG_NAME,MB_OK|MB_ICONSTOP);
		closesocket(sock);
		return 1;
	}

	/*Guardamos la tabla recibida en el lugar correspondiente*/
	i=0;
	while(mess[i]!='\0')	//hasta llegar al final del mensaje
	{
		for(j=0; mess[i]!=' '; i++)	//leemos el numero de la maquina (hasta el ' ')
			buff1[j++]=mess[i];
		buff1[j]='\0';
		for(i+=1, j=0; mess[i]!='\n'; i++)	//leemos su costo (hasta el '\n')
			buff2[j++]=mess[i];
		buff2[j]='\0';
		t[atoi(buff1)-1]=atoi(buff2);//el costo se pone en la maquina correspondiente
		i++;	//se salta el '\n'
	}

	/*Terminacion*/
	if(closesocket(sock) == SOCKET_ERROR)
	{
		MessageBox(NULL,"Error al cerrar el socket!",PROG_NAME,MB_OK|MB_ICONSTOP);
		return 1;
	}
	return 0;
}

/*Escribe el resultado en un archivo*/
int writetofile(struct Ruta *r)
{
	FILE *t;
	char aux[32], val[16];
  int i, j;

	if((t=fopen("bellford.txt","wt")) == NULL)
	{
		MessageBox(NULL,"Error al crear el archivo de salida!",PROG_NAME,MB_OK|MB_ICONSTOP);
		return 1;
	}
	strcpy(aux,"Maquina Costo Trayectoria\n");
	fwrite(aux,sizeof(char),strlen(aux),t);
	for(i=0; i<MAXMAQS; i++)
	{
		if(r[i].rout[0]==NULL || r[i].cost==INF)
			continue;
		itoa(i+1,val,10);
		fwrite(val,sizeof(char),strlen(val),t);
		for(j=0; j<8-strlen(val); j++)
			fputs(" ",t);
		itoa(r[i].cost,val,10);
		fwrite(val,sizeof(char),strlen(val),t);
		for(j=0; j<6-strlen(val); j++)
			fputs(" ",t);
		val[0]='\0';
		for(j=0; r[i].rout[j]!=NULL; j++)
		{
			itoa(r[i].rout[j],aux,10);
			strcat(val,aux);
			strcat(val,"-");
		}
		val[strlen(val)-1]='\0';
		fwrite(val,sizeof(char),strlen(val),t);
		fputc('\n',t);
	}
	if(fclose(t) == EOF)
		return 1;
	return 0;
}

/*Javier Novoa C.*/
