/*
  BELFRDSR.C										mayo / 2001
  Servidor no orientado a conexion para proporcionar las tablas al programa de Bellman Ford
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "winsock.h"

#define WINSOCK_VER 0x0101
#define DEF_PROTO 0
#define RCV_FLG 0
#define SND_FLG 0

char PROG_NAME[32]="Bellman Ford - Servidor ";
const int MAQNUM=1;

void terminate(int);
int bellserv(void);

void main()
{
	char aux[8];
	WSADATA wsadata;

  strcat(PROG_NAME,itoa(MAQNUM,aux,10));
	if(WSAStartup(WINSOCK_VER,&wsadata))
	{
		MessageBox(NULL,"Error en la version de Winsock!",PROG_NAME,MB_OK|MB_ICONEXCLAMATION);
    terminate(1);
	}
  terminate(bellserv());
}

/*Termina el programa*/
void terminate(int excd)
{
	WSACleanup();
	exit(excd);
}

/*Servidor para proporcionar mi tabla o pedirsela a otra maquina y darla a quien la pida*/
int bellserv(void)
{
  int inito(FILE **, char *, SOCKADDR_IN *);

	FILE *mitabla;
	SOCKET sock;
	SOCKADDR_IN sockadd, from, to;
	int flen=sizeof(from), tlen=sizeof(to), port;
	char mensaje[1024], aux[32], buf[8], fn[16]="tab";
	int i, j;
	long pos;

	/*Inicializacion*/
	strcat(fn,itoa(MAQNUM,aux,10));
  strcat(fn,".txt");
	if((mitabla=fopen(fn,"rb")) == NULL)
	{
		MessageBox(NULL,"Error al abrir el archivo de la tabla!",PROG_NAME,MB_OK|MB_ICONSTOP);
		return 1;
	}
	sock=socket(PF_INET,SOCK_DGRAM,DEF_PROTO);	//creamos el socket
	if(sock==INVALID_SOCKET)
	{
		MessageBox(NULL,"Error al crear el socket!",PROG_NAME,MB_OK|MB_ICONSTOP);
    fclose(mitabla);
		return 1; 
	}
	fgets(aux,8,mitabla);	//leemos el primer renglon de mi tabla: el numero de mi puerto
	port=atoi(aux);
	sockadd.sin_family=AF_INET;					//definimos los parametros del socket
	sockadd.sin_port=htons(port);
	sockadd.sin_addr.S_un.S_addr=0;
	if(bind(sock,(struct sockaddr far *)&sockadd,sizeof(SOCKADDR_IN)) == SOCKET_ERROR)
	{											//atamos el socket
		MessageBox(NULL,"Error al atar el socket!",PROG_NAME,MB_OK|MB_ICONSTOP);
    fclose(mitabla);
    closesocket(sock);
  	return 1;
	}
	pos=ftell(mitabla);							//posicion luego de mi puerto

	while(1)
  {
		fseek(mitabla,pos,SEEK_SET);		//despues de mi puerto
		/*Espera recibir una peticion y la procesa (regresa mi tabla o pide la de otro para regresarla)*/
		if(recvfrom(sock,(char far *)mensaje,sizeof(mensaje),RCV_FLG,
								(struct sockaddr far *)&from,(int far *)&flen) == SOCKET_ERROR)
		{
  		MessageBox(NULL,"Error al recibir!",PROG_NAME,MB_OK|MB_ICONEXCLAMATION);
			fclose(mitabla);
	    closesocket(sock);
  		return 1;
		}
		if(mensaje[0] == 'Q')				//mensaje especial: terminar el servidor
		{
			MessageBox(NULL,"Listo!",PROG_NAME,MB_OK|MB_ICONINFORMATION);
			break;
		}
			//formato del mensaje de peticion de tabla: N1-N2-...-T,
			//	donde: Nn=nodos posteriores al mio, T=pide la tabla
		else if(mensaje[0]=='T')		//me pide la tabla
		{
			//Arma el mensaje leyendo mi tabla
  	  j=0;
			while(!feof(mitabla))				//nos habiamos quedado en el 2do. renglon, luego de mi puerto...
			{
				fgets(aux,32,mitabla);		//leemos el siguiente renglon de mi tabla
				if(feof(mitabla))
      		break;
				for(i=0; aux[i]!='\0' && aux[i]!=' '; i++)	//leemos el numero de cada maquina
					mensaje[j++]=aux[i];											//y lo escribimos en el mensaje
    	  mensaje[j++]=' ';
				for(i+=1; aux[i]!='\0' && aux[i]!=' '; i++)	//igual con su costo
					mensaje[j++]=aux[i];
				if(aux[i]=='\0')
				{
	  			MessageBox(NULL,"Mi tabla esta corrompida!",PROG_NAME,MB_OK|MB_ICONEXCLAMATION);
					return 1;
				}
				mensaje[j++]='\n';				//luego de cada renglon, un salto de linea
			}
	    mensaje[j]='\0';						//al final de todo, un fin de cadena
		}
		else									//me pide pedirle a otro su tabla
		{
			for(i=0; mensaje[i]!='\0' && mensaje[i]!='-'; i++)
				buf[i]=mensaje[i];	//obtenemos el numero del 1er nodo al que hay que pedirle la informacion
			if(mensaje[i]=='\0')
			{
  			MessageBox(NULL,"El mensaje de peticion es erroneo!",PROG_NAME,MB_OK|MB_ICONEXCLAMATION);
				fclose(mitabla);
	  	  closesocket(sock);
  			return 1;
			}
			buf[i]='\0';
			//Inicializamos 'to' con el siguiente nodo al que va mensaje (N1)
			if(inito(&mitabla,buf,&to))
	    {
				fclose(mitabla);
	 		  closesocket(sock);
				return 1;
			}
			//Recorre el mensaje hasta el siguiente nodo (N2) (luego del primer '-')
    	i++;												//apuntamos a donde empieza N2
			for(j=0; i<(sizeof(mensaje)-1); i++, j++)	//recorremos el mensaje desde N2 hacia el inicio
  	  	mensaje[j]=mensaje[i];
			//Manda peticion...
			if(sendto(sock,(char far *)mensaje,sizeof(mensaje),SND_FLG,
								(struct sockaddr far *)&to,tlen) == SOCKET_ERROR)
			{
				MessageBox(NULL,"Error al enviar el mensaje a N2!",PROG_NAME,MB_OK|MB_ICONEXCLAMATION);
				fclose(mitabla);
  	  	closesocket(sock);
				return 1;
			}
			//...y espera la respuesta desde N2
			if(recvfrom(sock,(char far *)mensaje,sizeof(mensaje),RCV_FLG,
									(struct sockaddr far *)&to,(int far *)&tlen) == SOCKET_ERROR)
			{
				MessageBox(NULL,"Error al recibir respuesta desde N2!",PROG_NAME,MB_OK|MB_ICONEXCLAMATION);
				fclose(mitabla);
				closesocket(sock);
		  	return 1;
			}
/*			if(memcmp(&to,&tmp,tlen))
			{
				MessageBox(NULL,"Se recibio un mensaje desde otro nodo!",PROG_NAME,MB_OK|MB_ICONEXCLAMATION);
				fclose(mitabla);
				closesocket(sock);
		  	return 1;
			}*/
			//NOTA: la respuesta ya sera un mensaje de tabla armado y solo falta mandarlo
		}

		//Manda el mensaje (armado o recibido) a from
		if(sendto(sock,(char far *)mensaje,sizeof(mensaje),SND_FLG,
							(struct sockaddr far *)&from,flen) == SOCKET_ERROR)
		{
			MessageBox(NULL,"Error al enviar el mensaje a N2!",PROG_NAME,MB_OK|MB_ICONEXCLAMATION);
			fclose(mitabla);
			closesocket(sock);
  		return 1;
		}
  }
	/*Terminacion*/
	if(fclose(mitabla) == EOF)
	 	return 1;
	if(closesocket(sock))
   	return 1;
	return 0;
}

/*Inicializacion de la estructura para el siguiente nodo al que hay que mandar el mensaje*/
int inito(FILE **f, char *b, SOCKADDR_IN *t)
{
  int i, j;
	char aux[32], buf[8];

	t->sin_family=AF_INET;
	while(!feof(*f))
	{
		fgets(aux,32,*f); //mitabla se quedo al iniciar el 1er renglon luego de mi puerto
		if(feof(*f))
		{
			MessageBox(NULL,"Mi tabla no describe al nodo pedido!",PROG_NAME,MB_OK|MB_ICONEXCLAMATION);
			return 1;
		}
		for(i=0; aux[i]!='\0' && aux[i]!=' '; i++)
			buf[i]=aux[i];		//obtenemos el numero del 1er nodo de mi tabla
		if(aux[i]=='\0')
		{
  		MessageBox(NULL,"Mi tabla esta corrompida!",PROG_NAME,MB_OK|MB_ICONEXCLAMATION);
			return 1;
		}
		buf[i]='\0';
		if(!strcmp(b,buf))	//buscamos en mi tabla el renglon que describe al nodo N1 (de buf1)
		{
			for(i=0; aux[i]!=' '; i++);	//al 1er espacio (luego del numero de nodo)
			for(i+=1; aux[i]!=' '; i++);//al segundo espacio (luego del costo)
			for(i+=1, j=0; aux[i]!='.'; j++, i++)//desde el primer digito de la direccion IP
				buf[j]=aux[i];
			buf[j]='\0';
			t->sin_addr.S_un.S_un_b.s_b1=atoi(buf);
			i++;												//apuntamos al segundo digito de la direccion IP
			for(j=0; aux[i]!='.'; j++, i++)
				buf[j]=aux[i];
			buf[j]='\0';
			t->sin_addr.S_un.S_un_b.s_b2=atoi(buf);
			i++;												//apuntamos al tercer digito de la direccion IP
			for(j=0; aux[i]!='.'; j++, i++)
				buf[j]=aux[i];
			buf[j]='\0';
			t->sin_addr.S_un.S_un_b.s_b3=atoi(buf);
			i++;												//apuntamos al cuarto digito de la direccion IP
			for(j=0; aux[i]!=' '; j++, i++)
				buf[j]=aux[i];
			buf[j]='\0';
			t->sin_addr.S_un.S_un_b.s_b4=atoi(buf);
			i++;												//apuntamos al puerto
			for(j=0; aux[i]!='\n'; j++, i++)
				buf[j]=aux[i];
			buf[j]='\0';
			t->sin_port=htons(atoi(buf));
			break;	//como ya lo encontramos, nos salimos
    }
	}
  return 0;
}

/*Javier Novoa C.*/
